#! /bin/sh
# next line not seen by tcl upon reentry by exec \
DYLD_LIBRARY_PATH=$MAPPER_LIBRARY_PATH exec wish8.5 $0 -- ${1+"$@"}
### previous inline env required b/c Mac10.11 won't pass DYLD_LIBRARY_PATH

############################################################################
# put up one help pop-up
############################################################################
### catch undef MAPPER_LIBRARY_PATH (runs Aqua tk, which can't do fullscreen)
if { [exec uname] == "Darwin" && ![info exists env(MAPPER_LIBRARY_PATH)] } {
  puts "helpstim: ### env var MAPPER_LIBRARY_PATH not set"
  puts "helpstim: to fix: pushd <mapperdir>; source MapperEnv.{c}sh; popd"
  exit
}

### colors/fonts
set bgcol     #c5c5bd
set entbgcol  #e9e9e9
set selbgcol  lightgreen
set ffont2  { arial -12 }
set pfont   { helvetica -14 bold }
set ffontb  { helvetica -12 bold }
set ffontbb { helvetica -15 bold }
set ffont   { helvetica 9 }
set bfont   { helvetica 9 bold }

### for helpwin search
set xcmd Alt; set xcmdname alt
if { [exec uname] == "Darwin" } { set xcmd Meta; set xcmdname cmd }

### (use tcl error checking)
proc isnum { val } {
  if [catch {set num [expr int($val)]} ] { return 0 } else { return 1 }
}

### help window, maybe containing image (loc=std/varnotfile)
proc helpwin { helpfile {cols 50} {rows 30} {loc std} } {
  global env ffont2 ffontbb pfont bgcol entbgcol selbgcol
  global find currtxtind xcmd xcmdname

  set varflag 0
  set boxlab HELP
  if {$loc == "std"} {
    set nm .help-$helpfile
    if [winfo exists .help-$helpfile] { raise .help-$helpfile; return }
    if [string match /* $helpfile] {
      set boxlab "$boxlab  ([file tail $helpfile])"
    } else {
      set boxlab "$boxlab  ($helpfile)"
      set helpfile $env(MAPPER_DIR)/lib/help/$helpfile
    }
  } elseif {$loc == "varnotfile"} {
    set nm .help
    while {[info commands $nm] == "$nm"} {set nm ${nm}x}
    set varflag 1
  } else {
    set nm .help-$helpfile
    if [winfo exists .help-$helpfile] { raise .help-$helpfile; return }
    set boxlab "$boxlab  ($helpfile)"
    set helpfile $loc/$helpfile
  }
  if {!$varflag} { if [catch {open $helpfile} ret] { return } }
  set nm [replchar $nm "." "_"]        ;# strip periods b/c tcl winpath
  set nm [string replace $nm 0 0 "."]  ;# restore init period

  set f [toplevel $nm -borderwidth 10]
  #positionpopup $f
  wm resizable $f 0 1
  frame $f.ti -bg $bgcol
  frame $f.ti.l -bg $bgcol
  frame $f.ti.r -bg $bgcol
  pack $f.ti -side top -anchor w -fill x
  pack $f.ti.l -side left -anchor w -fill x
  pack $f.ti.r -side right
  label $f.ti.l.la -text $boxlab -font $ffontbb -bd 2 -relief groove \
    -padx 7 -pady 7 -bg $entbgcol
  pack $f.ti.l.la -side left -anchor w
  label $f.ti.r.la -text "(find: $xcmdname-F)" -font $ffont2 -bg $bgcol -pady 0
  label $f.ti.r.la2 -text "(next: $xcmdname-G)" -font $ffont2 -bg $bgcol -pady 0
  pack $f.ti.r.la $f.ti.r.la2 -side top -pady 0

  frame $f.t
  text $f.t.txt -width $cols -height $rows -font $ffont2 \
    -borderwidth 3 -relief flat -setgrid true -selectbackground $selbgcol \
    -yscrollcommand "$f.t.scroll set" -highlightbackground black -bg white \
    -inactiveselectbackground $selbgcol -selectforeground black
  scrollbar $f.t.scroll -command "$f.t.txt yview" -highlightbackground $bgcol
  pack $f.t.scroll -side right -fill y
  pack $f.t.txt -side left -expand true -fill y
  pack $f.t -side top -expand true -fill y
  $f.t.txt config -state normal
  if {$varflag} {
    $f.t.txt insert end $helpfile
  } else {
    set id [open $helpfile]
    $f.t.txt insert end [read $id]
    close $id
  }
  set b [frame $f.but -borderwidth 1]
  pack $b -side right
  # N.B.: must also destroy . to avoid stranded bg proc!
  button $b.ok -text "OK" -font $pfont -command "destroy $f .onefindpanel ." \
    -highlightbackground $bgcol -bg $entbgcol -padx 10 -pady 1
  pack $b.ok -side right -padx 5 -anchor e
  foreach win "$f $f.but $f.t.scroll" { $win config -bg $bgcol }
  bind $nm <Return> "destroy $f .onefindpanel ."
  bind $nm <Control-c> "destroy $f .onefindpanel ."
  set currtxtind 1.0  ;# init search to top, at help panel create
  set find ""         ;# init search patt
  bind $f.t.txt <$xcmd-f> "onefindpanel $f.t.txt"
  wm protocol $f WM_DELETE_WINDOW "destroy $f .onefindpanel ."
  focus $f.t.txt
  $f.t.txt mark set insert 1.0  ;# so xcmd-f works immediately
}

### where to put all popups (relative to root)
proc positionpopup { win {size {}} } {
  set winx [expr [winfo rootx .]+750]
  set winy [expr [winfo rooty .]-5]
  wm geometry $win $size+${winx}+${winy}
  return $size+${winx}+${winy}
}

### replace all instances of a char
proc replchar { str oldc newc } {
  set i 0
  set len [string length $str]
  set newstr ""
  while {$i < $len} {
    set c [string index $str $i]
    if {$c == $oldc} {
      set newstr "${newstr}${newc}"
    } else {
      set newstr "${newstr}${c}"
    }
    incr i
  }
  return $newstr
}

### one find panel (make, else just raise/move/reattach-to-new-textwidget)
proc onefindpanel { tw } {
  global find ffont2 ffontb bgcol entbgcol selbgcol xcmd

  set f .onefindpanel
  set geom +[expr [winfo rootx $tw] + 115]+[expr [winfo rooty $tw] - 61]
  if ![winfo exists $f] {
    toplevel $f -borderwidth 5 -bg $bgcol
    wm geometry $f $geom  ;# here, too, to avoid init corner flash
    wm title $f ""
    label $f.la -text "find:" -font $ffont2 -bg $bgcol
    entry $f.e -width 23 -font $ffont2 -textvariable find \
      -background $entbgcol -selectbackground $selbgcol -selectforeground black
    button $f.bu -text NEXT -font $ffontb -pady 2 -padx 11
    pack $f.la $f.e $f.bu -side left -padx 3
    wm resizable $f 0 0
  }
  raise $f
  wm geometry $f $geom
  $f.bu config -command "findnext $tw"
  bind $f <$xcmd-g> "findnext $tw"
  bind $f.e <Return> "findnext $tw"
  focus $f.e
}

### find/color next occurrence (pattern in global $find) for curr help panel
proc findnext { tw } {   ;# pass existing text widget
  global find currtxtind

  set cnt 0
  set ind [$tw search -count cnt $find $currtxtind]  ;# ignores cursor
  if {$cnt} {
    set currend [$tw index "$ind + $cnt c"]  ;# end of found
    $tw tag remove sel 0.0 end               ;# clear all
    $tw tag add sel $ind $currend            ;# highlight new
    $tw see $ind                             ;# move to see it
    set currtxtind $currend
  }
}

### help
if { [llength $argv] < 1 || [llength $argv] > 1 } {
  puts "use: helpstim <mapper-stimtype-number>"
  exit
} else {
  set stimtype [lindex $argv 0]
  if ![isnum $stimtype] {
    puts "helpstim: ### bad stimtype: $stimtype (not a number)"
    exit
  } else {
    set helpfile s${stimtype}
  }
}

### open helpwin
wm withdraw .
helpwin $helpfile 50 42

