#! /bin/csh -f
### run: Mac OS X csh script to setup USB port, pass skippulses to stimscript

### defaults
set pg = run
set initUSBprog = AccesLoader
set usbdev = USB-DIO-32
set skipUSBcheck = 0
set restype = XGA
set exptfile = hkexpt.scn
set bgimgfile = ""
set vertdisp = 0.0
set interocdist = def
set startstimscale = 1.0
set fivekeytrigger = 0
set apostrophetrigger = 0
set toggletrigger = 0
set nosetupusb = 0

### help
if ($#argv < 2) then
  echo ""
  echo \
    "Use: $pg <stimscript> <numskip> [-opts... see below]"
  echo ""
  echo "   first plug in USB trigger cable (if starting w/USB)"
  echo "   checks if USB $initUSBprog needs to run (no re-run)"
  echo -n "   $initUSBprog and <stimscript> must be on"; echo ' $path'
  echo "   numskip=waitF1 -> wait for Fn-F1 to start (no USB)"
  echo "   numskip=immed  -> start immediately (no USB)"
  echo "   numskip=0      -> wait for first pulse to start"
  echo "   numskip=1      -> skip 1, start on 2nd pulse"
  echo "   numskip=2      -> skip 2, start on 3rd pulse"
  echo "     [etc]"
  echo "     [same if trigger instead w/'5' key (-fivekeytrigger)]"
  echo "     [same if trigger instead w/apostrophe key (-apostrophetrigger)]"
  echo "   Fn-F1 starts (or restarts)"
  echo "   Fn-F2 resets to wait screen"
  echo "   opts: -QVGA   => 320x240"
  echo "         -VGA    => 640x480"
  echo "         -XGA-   => 1024x500"
  echo "         -XGA    => 1024x768"
  echo "         -WXGA   => 1440x900"
  echo "         -SXGA+  => 1400x1050"
  echo "         -SXGA++ => 1440x1080"
  echo "         -WSXGA  => 1600x1024"
  echo "         -WSXGA+ => 1680x1050"
  echo "         -HDMI   => 1920x1080"
  echo "         -WUXGA  => 1920x1200"
  echo "         -WQXGA  => 2560x1600"
  echo "         --------------------"
  echo "         -fivekeytrigger       [USB keystroke instead of USB-DIO-32]"
  echo "         -apostrophetrigger    [USB keystroke instead of USB-DIO-32]"
  echo "         -toggletrigger        [else default: pulse about 4ms wide]"
  echo "         -nosetupusb           [for testing, overrides req'd setup]"
  echo "         -vertdisp <-0.1 to 0.1>"
  echo "         -interocdist <~1.5>"
  echo "         -startstimscale <~1>"
  echo "         --------------------"
  echo "         -exptfile => only relevant for stimscript 'hkexpt'"
  echo "         -bgimgfile => only relevant for stimscript 'hkexpt'"
  echo ""
  exit
endif

### args
set stimscript = $argv[1] 
set numskip = $argv[2]
if (`echo $numskip | egrep '[^-0-9]|.-'` != "") then
  if ("$numskip" != "waitF1" && "$numskip" != "immed") then
    echo "${pg}: ### bad value for numskip: $numskip"
    exit
  endif
endif
if ($numskip == waitF1) set numskip = -2
if ($numskip == immed)  set numskip = -1
@ pulsecnt = $numskip + 1
if ($pulsecnt == -1 || $pulsecnt == 0) set skipUSBcheck = 1
shift; shift
set foundexptfile = 0
set foundbgimgfile = 0
set foundvertdisp = 0
set foundinterocdist = 0
set foundstartstimscale = 0
set i = 1
while ($i <= $#argv)
  if ("$argv[$i]" == "-QVGA")   set restype = QVGA
  if ("$argv[$i]" == "-VGA")    set restype = VGA
  if ("$argv[$i]" == "-XGA")    set restype = XGA
  if ("$argv[$i]" == "-XGA-")   set restype = XGA-
  if ("$argv[$i]" == "-hires")  set restype = SXGA+
  if ("$argv[$i]" == "-WXGA")   set restype = WXGA
  if ("$argv[$i]" == "-SXGA+")  set restype = SXGA+
  if ("$argv[$i]" == "-SXGA++") set restype = SXGA++
  if ("$argv[$i]" == "-WSXGA")  set restype = WSXGA
  if ("$argv[$i]" == "-WSXGA+") set restype = WSXGA+
  if ("$argv[$i]" == "-HDMI")   set restype = HDMI
  if ("$argv[$i]" == "-WUXGA")  set restype = WUXGA
  if ("$argv[$i]" == "-WQXGA")  set restype = WQXGA
  if ("$argv[$i]" == "-fivekeytrigger") then
    set skipUSBcheck = 1
    set fivekeytrigger = 1
  endif
  if ("$argv[$i]" == "-apostrophetrigger") then
    set skipUSBcheck = 1
    set apostrophetrigger = 1
  endif
  if ("$argv[$i]" == "-toggletrigger") set toggletrigger = 1
  if ("$argv[$i]" == "-nosetupusb") set nosetupusb = 1
  # get opt arg if opt found last time
  if ($foundexptfile) then
    set exptfile = $argv[$i]
    set foundexptfile = 0
  endif
  if ($foundbgimgfile) then
    set bgimgfile = $argv[$i]
    set foundbgimgfile = 0
  endif
  if ($foundvertdisp) then
    set vertdisp $argv[$i]
    set foundvertdisp = 0
  endif
  if ($foundinterocdist) then
    set interocdist = $argv[$i]
    set foundinterocdist = 0
  endif
  if ($foundstartstimscale) then
    set startstimscale = $argv[$i]
    set foundstartstimscale = 0
  endif

  # set flags to get arg next time
  if ("$argv[$i]" == "-exptfile") set foundexptfile = 1
  if ("$argv[$i]" == "-bgimgfile") set foundbgimgfile = 1
  if ("$argv[$i]" == "-vertdisp") set foundvertdisp = 1
  if ("$argv[$i]" == "-interocdist") set foundinterocdist = 1
  if ("$argv[$i]" == "-startstimscale") set foundstartstimscale = 1
  if ("$argv[$i]" == "-exptfile") set foundexptfile = 1
  if ("$argv[$i]" -= "-bgimgfile") set foundbgimgfile = 1
  @ i++
end

### errs
foreach bin ($initUSBprog $stimscript)
  if ($bin == "$initUSBprog" && $skipUSBcheck) continue
  if (`which $bin` !~ *$bin) then
    echo "${pg}: ### $bin not found"
    exit
  endif
end
if (`uname -s` == "Darwin" && $term != "xterm") then
  #echo "${pg}: ### pre-10.6 run from xterm in X11.app"
  echo "${pg}: ### N.B.: easier to kill from xterm in X11.app on MacOS 10.6+"
  #exit
endif

### run AccesLoader if needed (N.B.: echo thru allscripts only at stim start)
if (! $skipUSBcheck) then
  set cnt = `system_profiler SPUSBDataType -detailLevel mini | grep -c $usbdev`
  if ($cnt == 0) then
    echo "${pg}: TTL->USB box ($usbdev) not initialized"
    echo "${pg}: `which $initUSBprog`"
    echo "${pg}: plug in trigger USB cable now (if not already in)"
    set ret = `$initUSBprog`
    echo "${pg}: $ret"
    sleep 1       # don't remove
  else
    echo "${pg}: TTL->USB box ($usbdev) already initialized"
  endif
endif

### pass display res, pulsecnt args to stim scripts via env
if ($restype == "QVGA") then
  setenv STIMWINX 320; setenv STIMWINY  240
endif
if ($restype == "VGA") then
  setenv STIMWINX 640; setenv STIMWINY  480
endif
if ($restype == "XGA-") then
  setenv STIMWINX 1024; setenv STIMWINY  500
endif
if ($restype == "XGA") then
  setenv STIMWINX 1024; setenv STIMWINY  768
endif
if ($restype == "WXGA") then
  setenv STIMWINX 1440; setenv STIMWINY 900
endif
if ($restype == "SXGA+") then
  setenv STIMWINX 1400; setenv STIMWINY 1050
endif
if ($restype == "SXGA++") then
  setenv STIMWINX 1440; setenv STIMWINY 1080
endif
if ($restype == "WSXGA") then
  setenv STIMWINX 1600; setenv STIMWINY 1024
endif
if ($restype == "WSXGA+") then
  setenv STIMWINX 1680; setenv STIMWINY 1050
endif
if ($restype == "HDMI") then
  setenv STIMWINX 1920; setenv STIMWINY 1080
endif
if ($restype == "WUXGA") then
  setenv STIMWINX 1920; setenv STIMWINY 1200
endif
if ($restype == "WQXGA") then
  setenv STIMWINX 2560; setenv STIMWINY 1600
endif
setenv PULSECNT $pulsecnt
if ($stimscript =~ *hkexpt) then
  setenv EXPTFILE $exptfile
  setenv BGIMGFILE $bgimgfile
endif
if ($pulsecnt == -1) echo "${pg}: wait for F1 key to start"
if ($pulsecnt == 0)  echo "${pg}: start stim immediately"
if ($pulsecnt > 0)   echo "${pg}: start on pulse $pulsecnt (skip $numskip)"
sleep 1
### 2022: curr Nvidia def=1, but need shell override to 0 to fix jumpy CentOS7!
#if (`uname` == "Linux") setenv __GL_SYNC_TO_VBLANK 1   # nvidia driver

### pass other options by "mapper scripts accept appended options" method
set opts = ""
if ($vertdisp != "0.0") set opts = "$opts -vertdisp $vertdisp"
if ($interocdist != "def") set opts = "$opts -interocdist $interocdist"
if ($startstimscale != "1.0") set opts = "$opts -startstimscale $startstimscale"
if ($fivekeytrigger) set opts = "$opts -fivekeytrigger"
if ($apostrophetrigger) set opts = "$opts -apostrophetrigger"
if ($toggletrigger) set opts = "$opts -toggletrigger"
if ($nosetupusb) set opts = "$opts -nosetupusb"

### run script
echo "${pg}: exec $stimscript $opts"
exec $stimscript $opts

